# models.py
from flask_sqlalchemy import SQLAlchemy
from flask_login import UserMixin
db = SQLAlchemy()

class User(db.Model,UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(80), unique=True, nullable=False)
    password = db.Column(db.String(120), unique=False, nullable=False)
    is_su = db.Column(db.Boolean, unique=False, nullable=False,default=False)
    is_member = db.Column(db.Boolean, unique=False, nullable=False,default=False)
    real_name = db.Column(db.String(80), unique=False, nullable=True)
    profile_file = db.Column(db.String(120), unique=False, nullable=True)

class Seasons(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    season_name = db.Column(db.String(80), unique=False, nullable=True)
    start_time = db.Column(db.Date, unique=False, nullable=False)
    end_time = db.Column(db.Date, unique=False, nullable=True)
    enable = db.Column(db.Boolean, unique=False, nullable=False,default=False)

class Games(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    game_date = db.Column(db.DateTime, unique=False, nullable=False)
    # player1_id = db.Column(db.Integer, unique=False, nullable=False)
    player1_id = db.Column(db.Integer,db.ForeignKey('user.id'), unique=False, nullable=False)
    player1 = db.relationship("User",foreign_keys=[player1_id],uselist=False)
    score1 = db.Column(db.SmallInteger, unique=False, nullable=False)
    # player2_id = db.Column(db.Integer, unique=False, nullable=False)
    player2_id = db.Column(db.Integer,db.ForeignKey('user.id'), unique=False, nullable=False)
    player2 = db.relationship("User",foreign_keys=[player2_id],uselist=False)
    score2 = db.Column(db.SmallInteger, unique=False, nullable=False)
    enable = db.Column(db.Boolean, unique=False, nullable=False,default=False)
    # laucher_id = db.Column(db.Integer, unique=False, nullable=False)
    laucher_id = db.Column(db.Integer,db.ForeignKey('user.id'), unique=False, nullable=False)
    laucher = db.relationship("User",foreign_keys=[laucher_id],uselist=False)
    lauch_date = db.Column(db.DateTime, unique=False, nullable=True)
    # confirmer_id = db.Column(db.Integer, unique=False, nullable=True)
    confirmer_id = db.Column(db.Integer,db.ForeignKey('user.id'), unique=False, nullable=False)
    confirmer = db.relationship("User",foreign_keys=[confirmer_id],uselist=False)
    confirm_date = db.Column(db.DateTime, unique=False, nullable=True)
    season_id = db.Column(db.Integer,db.ForeignKey('seasons.id'), unique=False, nullable=False)
    season = db.relationship("Seasons",foreign_keys=[season_id],uselist=False)
    game_type = db.Column(db.Integer, unique=False, nullable=False)





class SeasonPlayers(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    player_id = db.Column(db.Integer,db.ForeignKey('user.id'), unique=False, nullable=False)
    player = db.relationship("User",foreign_keys=[player_id])
    season_id = db.Column(db.Integer,db.ForeignKey('seasons.id'), unique=False, nullable=False)
    season = db.relationship("Seasons",foreign_keys=[season_id])
    score = db.Column(db.Integer, unique=False, nullable=False,default=0)
    group = db.Column(db.Integer, unique=False, nullable=False)


